from .dmoz_cat import dmoz_cat
from .phish_cat import phish_cat
from .shalla_cat import shalla_cat
from .pred_phish import pred_phish
from .pred_shalla import pred_shalla
from .pred_toulouse import pred_toulouse
from .pred_malware import pred_malware

__all__ = ['dmoz_cat', 'phish_cat', 'shalla_cat',
           'pred_phish', 'pred_shalla', 'pred_toulouse', 'pred_malware']
